@@ -6,15 +6,8 @@
  * mpage:    a program to reduce pages of print so that several pages
  *           of output appear on one printed page.
  *
- * Written by:
- *   ...!uunet!\                       Mark Hahn, Sr Systems Engineer
- *              >pyrdc!mark            Pyramid Technology Corporation
- * ...!pyramid!/                       Vienna, Va    (703)848-2050
- *
- *
+ * Copyright (c) 1994-1999 Marcel J.E. Mol, The Netherlands
  * Copyright (c) 1988 Mark P. Hahn, Herndon, Virginia
- * Copyright (c) 1994-1997 Marcel J.E. Mol, The Netherlands
- *                    marcel@mesa.nl
  *  
  *     Permission is granted to anyone to make or distribute verbatim
  *     copies of this document as received, in any medium, provided
@@ -31,21 +24,27 @@
 /*
  * Function declarations
  */
-static char **slice();
+static char **slice(char *string, int *cntp);
 
 
 
 int
-do_args(argc, argv, envflag)
- int argc;
- char **argv;
- int envflag;
+do_args(int argc, char **argv, int envflag)
 {
     char *optstr;
     int consumed;
     int currarg;
     int opterrors;
 
+    int i;
+
+    opt_jarg = 0;
+    for (i = 0; i < MAXJARG; i++) {
+        opt_first[i] = 1;
+        opt_last[i]  = MAXINT;
+        opt_alt[i]   = 1;
+    }
+
 #define OPTARG()  \
     { consumed = 1; if(*++optstr == '\0') optstr = argv[++currarg]; }
 
@@ -111,9 +110,9 @@ do_args(argc, argv, envflag)
                             }
                             else
                                 sign = 1;
-                            if (isdigit(*optstr)) {
+                            if (isdigit((int)*optstr)) {
                                 pb = *optstr - '0';
-                                while (*++optstr && isdigit(*optstr))
+                                while (*++optstr && isdigit((int)*optstr))
                                     pb = pb*10 + *optstr - '0';
                             }
                             pb *= sign;
@@ -177,6 +176,10 @@ do_args(argc, argv, envflag)
                         OPTARG();
                         dateformat = optstr;
                         break;
+                case 'e': /* GPN. for coli, 2,3,4,1(all) pages */
+                        Coli = 3;
+                        opt_duplex = 1 - opt_duplex;
+                        break;
 	        case 'E': /* GPN. for coli, 2,3(inside) pages */
 	                Coli = 2;
                         break;
@@ -200,16 +203,27 @@ do_args(argc, argv, envflag)
                         break;
                 case 'j':    /* Just these sheets */
                         OPTARG();
-                        opt_first = isdigit(*optstr) ? 
+                        if (opt_jarg >= MAXJARG) {
+                            fprintf(stderr,
+                                    "%s: to many j args (%d), ignoring %s\n", 
+                                    MPAGE, MAXJARG, optstr);
+                            break;
+                        }
+                        opt_first[opt_jarg] = isdigit((int)*optstr) ? 
                                     strtol(optstr, &optstr, 10) : 1;
                         if (*optstr == '-') {
-                            if (isdigit(*++optstr))
-                                opt_last = strtol(optstr, &optstr, 10);
+                            if (isdigit((int)*++optstr))
+                                opt_last[opt_jarg] = strtol(optstr, &optstr, 10);
                         }
                         else
-                            opt_last = MAXINT;
+                            opt_last[opt_jarg] = MAXINT;
                         if (*optstr == '/' || *optstr == '%')
-                            opt_alt = atoi(++optstr);
+                            opt_alt[opt_jarg] = atoi(++optstr);
+                        opt_jarg++;
+                        break;
+                case 'J':
+                        OPTARG();
+                        ps_pagenum = atoi(optstr) - 1;
                         break;
                 case 'k':    /* toggle kill on trailer */
                         opt_killtrail = 1 - opt_killtrail;
@@ -244,9 +258,9 @@ do_args(argc, argv, envflag)
                             }
                             else
                                 sign = 1;
-                            if (isdigit(*optstr)) {
+                            if (isdigit((int)*optstr)) {
                                 sm = *optstr - '0';
-                                while (*++optstr && isdigit(*optstr))
+                                while (*++optstr && isdigit((int)*optstr))
                                     sm = sm*10 + *optstr - '0';
                             }
                             sm *= sign;
@@ -296,9 +310,9 @@ do_args(argc, argv, envflag)
                             }
                             else
                                 sign = 1;
-                            if (isdigit(*optstr)) {
+                            if (isdigit((int)*optstr)) {
                                 pm = *optstr - '0';
-                                while (isdigit(*++optstr))
+                                while (isdigit((int)*++optstr))
                                     pm = pm*10 + *optstr - '0';
                             }
                             pm *= sign;
@@ -341,12 +355,12 @@ do_args(argc, argv, envflag)
                 case 'P':    /* Printer */
                         consumed = 1;
                         doprint = 1;
-                        if (*++optstr)
+                        if (*++optstr) {
                             if (*optstr == '-' && *(optstr+1) == '\0')
                                 doprint = 0; /* kill MPAGE envvar que setting*/
-                            else {
+                            else
                                 printque = optstr;
-                            }
+                        }
                         break;
                 case 'r':
                         opt_reverse = 1;
@@ -418,7 +432,7 @@ do_env()
 {
     int argc;
     char **argv;
-    char copy[LINESIZE];
+    char *copy;
     char *env;
 
 #if SPOOLER == ATT_SPOOLER
@@ -432,7 +446,11 @@ do_env()
         libdir = env;
 
     if ((env = getenv("MPAGE")) != NULL) {
-        strcpy(copy, env);
+        if ((copy = strdup(env)) == NULL) {
+            fprintf(stderr, "%s: Unable to alloc memory for environment args\n",
+                    MPAGE);
+            return -1;
+        }
         argv = slice(copy, &argc);
         if (do_args(argc, argv, 1) < 0) {
             fprintf(stderr, "%s: error in environment \"%s\"\n", MPAGE, env);
@@ -449,9 +467,7 @@ do_env()
 char *slc_argv[ARGCNT+1];
 
 static char **
-slice(string, cntp)
- char *string;
- int *cntp;
+slice(char *string, int *cntp)
 {
     int count;
 
@@ -492,7 +508,7 @@ slice(string, cntp)
              * drops us out of all the loops
              */
             if (*string) {
-                *string = 0;
+                *string = '\0';
                 string++;
             }
         }
