@@ -6,15 +6,8 @@
  * mpage:	a program to reduce pages of print so that several pages
  * 	  	of output appear on one printed page.
  *
- * Written by:
- *   ...!uunet!\                       Mark Hahn, Sr Systems Engineer
- *              >pyrdc!mark            Pyramid Technology Corporation
- * ...!pyramid!/                       Vienna, Va    (703)848-2050
- *
- *
+ * Copyright (c) 1994-1999 Marcel J.E. Mol, The Netherlands
  * Copyright (c) 1988 Mark P. Hahn, Herndon, Virginia
- * Copyright (c) 1994-1997 Marcel J.E. Mol, The Netherlands
- *                    marcel@mesa.nl
  *  
  *     Permission is granted to anyone to make or distribute verbatim
  *     copies of this document as received, in any medium, provided
@@ -50,6 +43,9 @@ struct page_desc paper[] = {
     { "Legal",        612, 1008 },
     { "Statement",    396,  612 },
     { "Executive",    540,  720 },
+    { "A0",           2384, 3368 },  /* 840 x 1188 mm */
+    { "A1",           1684, 2384 },  /* 594 x 840 mm */
+    { "A2",           1192, 1684 },  /* 420 x 594 mm */
 /*    { "A3",           842, 1190 }, */
     { "A3",           842, 1192 },  /* 297 x 420 mm */
 /* ISO 216 conforming says 595x841 ... */
@@ -86,18 +82,25 @@ struct pagepoints two_normal[] = {
 
 /* GPN outside 2 pages */
 struct pagepoints two_normal_co[] =   {
-	   {xbase1, ytop2, 0},	{0, 0, 1},
-	   {0,  0,  1},		{xbase1, ytop4, 0},
+	   {xbase1, ytop2, 0},	{0, 0, SKIP_PS},
+	   {0,  0,  SKIP_PS},		{xbase1, ytop4, 0},
 	   {0,  0,  0}
 
 };
 /* GPN. inside 2 pages */
 struct pagepoints two_normal_ci[] =   {
-	   {0, 0, 1},		{xbase1, ytop4, 0},
-	   {xbase1, ytop2, 0},	{0, 0, 1},
+	   {0, 0, SKIP_PS},		{xbase1, ytop4, 0},
+	   {xbase1, ytop2, 0},	{0, 0, SKIP_PS},
 	   {0,  0,  0}
 };
 
+/* GPN. all 4 pages */
+struct pagepoints four_normal_dm[] =   {
+           {xbase1, ytop2, STORE_PS}, {xbase1, ytop4, 0},
+           {xbase1, ytop2, 0},  {0, 0, 0}, {xbase1, ytop4, FLUSH_PS},
+           {0,  0,  0}
+};
+
 /* base points for four pages, normal aspect, running reduced pages
  * read from left to right */
 struct pagepoints lr_four_normal[] = {
@@ -270,7 +273,8 @@ struct sheet land_ud_normal[] = {
 /* GPN. sheet */
 struct sheet coli [] =   {
   	/* 1 */ { 80, 66, yht2,  xwid1, -90, outline_2, two_normal_co },
-  	/* 1 */ { 80, 66, yht2,  xwid1, -90, outline_2, two_normal_ci },
+  	/* 2 */ { 80, 66, yht2,  xwid1, -90, outline_2, two_normal_ci },
+        /* 3 */ { 80, 66, yht2,  xwid1, -90, outline_2, four_normal_dm },
 };
 
 /* array of sheet lists for left to right reading printed pages */
@@ -323,7 +327,6 @@ int opt_tabstop       = DEFAULTTABSTOP;	/* width of a tab */
 int opt_lines         = 0;		/* lines to fit on reduced page */
 int opt_killtrail     = 1;		/* Quit reading input on %%TRailer */
 int opt_width         = 0;	        /* columns to fit on reduced page */
-int opt_last          = MAXINT;	        /* print as many as supplied */
 char * opt_page       = PAGE_DEF;       /* default paper size */
 /* boolean's: set default to 0 or 1 */
 int opt_pr            = 0;		/* if true use pr(1) to format output */
@@ -334,14 +337,16 @@ int opt_outline       = 1;	        /* don't normally outline the pages */
 int opt_verbose       = 0;        /* print a count of pages produced */
 int opt_square        = 1;        /* print pages with natural aspect ratio */
 int opt_reverse       = 0;        /* by default print sheets in forward order */
-int opt_first         = 1;        /* start with first sheet */
-int opt_alt           = 1;        /* by default print all sheets, odd+even */
+int opt_jarg          = 0;        /* number of -j opt sets */
+int opt_first[MAXJARG];           /* start with first sheet per -j */
+int opt_last[MAXJARG];         	  /* print as many as supplied per -j */
+int opt_alt[MAXJARG];             /* by default print all sheets, odd+even per -j*/
 int opt_file          = 1;        /* should each file appera on a new sheet */
-int opt_duplex        = 0;        /* duplex mode flag */
+int opt_duplex        = DEFAULT_DUPLEX;  /* duplex mode flag */
 int opt_tumble        = 0;        /* tumble overy second pages */
 int opt_textbox       = 0;        /* don't normally draw box around text */
 int opt_input         = IN_AUTO;  /* select input file format */
-int opt_encoding      = DEFAULT_ENCODING;	/* use default encoding or not */
+int opt_encoding      = DEFAULT_ENCODING; /* use default encoding or not */
 struct pagebox textbox = {0, 0, 80, 66, 0};
 
 char * opt_header = NULL;      /* the header for pr's -h option */
@@ -361,7 +366,8 @@ char * sheethead  = "";	       /* Leave empty to get default sheetheader:
  * various global information
  */
 char MPAGE[]   = "mpage";   /* program name */
-int ps_pagenum = 0;	    /* current sheet count */
+int ps_pagenum = 0;	    /* current sheet count (printed or not) */
+int ps_outpages= 0;	    /* sheets printed */
 int had_ps     = 0;         /* did we process ps files */
 
 int first_encoding = -1; /* first encoding in character set */
@@ -378,7 +384,7 @@ void usage(int errcode)
  mpage - print multiple pages on postscript, version %s\n\n\
  mpage [-1248acEfHlkoOrRStvxX] [-b papersize] [-Btextboxmargin]\n\
        [-C [encodingfile]] [-da|p] [-D dateformat] [-F fontname]\n\
-       [-h header] [-I indent] [-j pagespec] [-L lines]\n\
+       [-h header] [-I indent] [-j pagespec] [-J startpageno] [-L lines]\n\
        [-msheetmargin] [-Mpagemargin] [-p[prprog]] [-P[printer]]\n\
        [-s tabstop] [-W width] [-X [header]] [-z printcmd] [-Z quearg]\n\
        [file...]]\n",
@@ -391,10 +397,10 @@ void usage(int errcode)
  -a Toggle across/updown layout (u)	-l Toggle portrait/landscape (p)\n\
  -f Toggle folding long lines (off)	-o Toggle printing outlines (on)\n\
  -r Reverse printing, last->first sheet	-v Toggle verbose output, (on)\n\
- -F Text font to be used (Courier)      -C Character encoding filename\n\
+ -F Text font to be used (%s)      -C Character encoding filename\n\
  -E Print every second and third page   -O Print every first and fourth page\n\
  -s Define tabstop width (%d)            -k kill on %%TRailer is PS file\n\
- -b papersize (%s), use l of ? to get a list of sizes\n\
+ -b papersize (%s), use l or ? to get a list of sizes\n\
  -R Switch to across mode, with first page at lower left corner\n\
  -H Create page header for each page for text files\n\
  -X Print physical page header (filename + physical pagenumber)\n\
@@ -410,11 +416,13 @@ void usage(int errcode)
  -Z Specify print command print queue option (%s).\n\
  -j Print specified sheets: first[-last][%%interval]\n\
     -j 1-10 does first 10 sheets, -j 1%%2 prints odd ones, -j 2%%2 even ones.\n\
- -t Toggle printing both sides of the paper (Duplex mode, off)\n\
+ -J Set the start of the sheet page count\n\
+ -t Toggle printing both sides of the paper (Duplex mode, %s)\n\
  -T Toggle tumble of every second pages when printing in duplex mode (off)\n",
-    opt_tabstop, PAGE_DEF, printprog, printarg
+    fontname, opt_tabstop, PAGE_DEF, printprog, printarg,
+    opt_duplex ? "on" : "off"
     );
-    fprintf(stderr, "\n(c) 1993-1998 Marcel Mol, marcel@mesa.nl (MESA Consulting)\n");
+    fprintf(stderr, "\n(c) 1993-2002 Marcel Mol, marcel@mesa.nl (MESA Consulting)\n");
 
     return;
 
