@@ -6,15 +6,8 @@
  * mpage:    a program to reduce pages of print so that several pages
  *           of output appear on one printed page.
  *
+ * Copyright (c) 1994-1999 Marcel J.E. Mol, The Netherlands
  * Copyright (c) 1988 Mark P. Hahn, Herndon, Virginia
- * Copyright (c) 1994-1997 Marcel J.E. Mol, The Netherlands
- *                    marcel@mesa.nl
-
- * Written by:
- *   ...!uunet!\                       Mark Hahn, Sr Systems Engineer
- *              >pyrdc!mark            Pyramid Technology Corporation
- * ...!pyramid!/                       Vienna, Va    (703)848-2050
- *
  *  
  *     Permission is granted to anyone to make or distribute verbatim
  *     copies of this document as received, in any medium, provided
@@ -28,6 +21,8 @@
 #include <string.h>
 #include <time.h>
 #include <fcntl.h>
+#include <locale.h>
+
 
 #include "encoding.h"
 
@@ -36,7 +31,7 @@
  */
 static void ps_title();
 
-
+char *current_locale;
 
 int
 main(argc, argv)
@@ -49,6 +44,7 @@ char **argv;
     struct sheet *thesheet;
     char outcommand[LINESIZE]; /* the command which is the output filter */
 
+    current_locale = setlocale(LC_ALL,"");
 
    
 #ifdef __EMX__
@@ -112,6 +108,8 @@ char **argv;
         thesheet = &(coli[0]);
     if (Coli == 2)
         thesheet = &(coli[1]);
+    if (Coli == 3)
+        thesheet = &(coli[2]);
 
     orientation = (sheetindex + sheetaspect) & 0x01;
 
@@ -169,10 +167,6 @@ char **argv;
      */
     fprintf(outfd, "%%%%Trailer\n");
     if (opt_verbose) {
-        /*
-        ps_pagenum++;
-        fprintf(outfd, "%%%%Page: %d %d\n", ps_pagenum, ps_pagenum);
-        */
         fprintf(outfd, "statusdict begin jobname print flush");
         fprintf(outfd, " (: Job finished:\\n) print\n");
         fprintf(outfd, "(\\tmpage time (s) = ) print flush usertime ");
@@ -180,9 +174,9 @@ char **argv;
         fprintf(outfd, " flush pagecount mp_pgc sub ==\nend flush\n");
         /* fprintf(outfd, "showpage\n"); */
     }
-    fprintf(outfd, "%%%%Pages: %d\n", ps_pagenum);
+    fprintf(outfd, "%%%%Pages: %d\n", ps_outpages);
     if (opt_verbose) {
-        fprintf(stderr, "[%s: %d pages, ", MPAGE, ps_pagenum);
+        fprintf(stderr, "[%s: %d pages, ", MPAGE, ps_outpages);
         if (doprint) {
             if (printque != NULL)
                 fprintf(stderr, "print queue %s]\n", printque);
@@ -237,6 +231,9 @@ ps_title(name, outfd)
     fprintf(outfd, "/mp_pgc statusdict begin pagecount end def\n");
     fprintf(outfd, "statusdict begin /jobname (%s) def end\n", name);
     if (opt_duplex) {
+       fprintf(outfd, "%%BeginFeature: *Duplex DuplexNoTumble\n");
+       fprintf(outfd, "<< /Duplex true /Tumble false >> setpagedevice");
+       fprintf(outfd, "%%EndFeature");
        fprintf(outfd, "statusdict /setduplexmode known");
        fprintf(outfd, " { statusdict begin true setduplexmode end } if\n");
        if (opt_tumble) {
@@ -244,6 +241,12 @@ ps_title(name, outfd)
            fprintf(outfd, "{ statusdict begin true settumble end } if\n"); 
        }
     }
+    else {
+       /* CAN WE DO THIS FOR PRINTERS THAT DO NOT NDERSTAND DUPLEX PRINTING??? */
+       fprintf(outfd, "statusdict /setduplexmode known");
+       fprintf(outfd, " { statusdict begin false setduplexmode end } if\n");
+    }
+
     if (opt_encoding) {
         fprintf(outfd,
                 "/reencsmalldict 12 dict def "
@@ -278,7 +281,7 @@ ps_title(name, outfd)
                 exit(1);
             }
             while (fgets(buf, LINESIZE, charfp) != NULL) {
-                if (*buf != '%' && *buf != '\n')
+                if (*buf != '%' && *buf != '\n') {
                     if (first_encoding == -1) {
                         first_encoding = atoi(buf);
                         last_encoding = atoi(strchr(buf, ' '));
@@ -289,6 +292,7 @@ ps_title(name, outfd)
                     }
                     else
                         fprintf(outfd, "%s", buf);
+                }
             }
         }
         else { /* use internal default encoding */
@@ -309,11 +313,22 @@ ps_title(name, outfd)
 
     fprintf(outfd, "/textfont /%s findfont %d scalefont def\n",
                    opt_encoding ? "OurCharSet" : fontname, fsize - 1);
+    fprintf(outfd, "/textfontbold /%s-Bold findfont %d scalefont def\n",
+                   opt_encoding ? "OurCharSet" : fontname, fsize - 1);
     fprintf(outfd, "/fnamefont /Times-Bold findfont %d scalefont def\n", HSIZE);
     fprintf(outfd, "/headerfont /Times-Bold findfont %d scalefont def\n",
                    HSIZE - 2);
     fprintf(outfd, "textfont setfont\n");
     fprintf(outfd, "(a) stringwidth pop /mp_a_x exch def\n");
+#ifdef KANJI
+#ifdef GOTHIC
+    fprintf(outfd, "/kanj /GothicBBB-Medium-H findfont %d scalefont def\n", fsize - 1);
+    fprintf(outfd, "/han /GothicBBB-Medium.Hankaku findfont %d scalefont def\n", fsize - 1);
+#else
+    fprintf(outfd, "/kanj /Ryumin-Light-H findfont %d scalefont def\n", fsize - 1);
+    fprintf(outfd, "/han /Ryumin-Light.Hankaku findfont %d scalefont def\n", fsize - 1);
+#endif
+#endif
 # ifdef DEBUG
     if (Debug_flag & DB_PSMPAGE)
         fprintf(outfd, "(\\t'a' length ) print mp_a_x ==\nflush\n");
