@@ -6,15 +6,9 @@
  * mpage:	A program to reduce pages of print so that several pages
  * 	  	of output appear on one sheet of paper.
  *
+ * Copyright (c) 1994-1999 Marcel J.E. Mol, The Netherlands
  * Copyright (c) 1988 Mark P. Hahn, Herndon, Virginia
- * Copyright (c) 1994-1997 Marcel J.E. Mol, The Netherlands
- *                    marcel@mesa.nl
  *  
- * Written by:
- *   ...!uunet!\                       Mark Hahn, Sr Systems Engineer
- *              >pyrdc!mark            Pyramid Technology Corporation
- * ...!pyramid!/                       Vienna, Va    (703)848-2050
- *
  *
  *     Permission is granted to anyone to make or distribute verbatim
  *     copies of this document as received, in any medium, provided
@@ -34,37 +28,56 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <limits.h>
+#include <string.h>
 
-# define	VERSION		"2.5.1pre2 March 1998"
+#define	VERSION		"2.5.3 Oktober 2002"
 
-# define	TRUE		1
-# define	FALSE		0
+#define	TRUE		1
+#define	FALSE		0
 
-# define	LINESIZE	1024
+#define	LINESIZE	1024
 
-# define	FILE_EOF	1
-# define	FILE_MORE	2
+#define	FILE_CONT	0
+#define	FILE_EOF	1
+#define	FILE_MORE	2
 
-# define	LINE_MORE	5
-# define	LINE_EOF_NOW	4
-# define	LINE_EOF	3
-# define	LINE_OVERPRINT	2
-# define	LINE_BLANK	1
+#define	LINE_MORE	5
+#define	LINE_EOF_NOW	4
+#define	LINE_EOF	3
+#define	LINE_OVERPRINT	2
+#define	LINE_BLANK	1
 
-# define	TSIZE		12
-# define 	HSIZE		TSIZE+2
+#define SKIP_PS		1
+#define STORE_PS	2
+#define FLUSH_PS	3
+
+#define	TSIZE		12
+#define HSIZE		TSIZE+2
+
+#define DEFAULTPMARGIN  4
 
-# define       DEFAULTPMARGIN  4
 #if !defined(DEFAULTSMARGIN)
-# define        DEFAULTSMARGIN  20
+# define       DEFAULTSMARGIN  20
 #endif
-# define        DEFAULTTMARGIN  0
-# define        DEFAULTTABSTOP  8
-# define	MAXINT		(1 << 30)
+
+#define DEFAULTTMARGIN  0
+#define DEFAULTTABSTOP  8
+
+#if !defined(MAXINT)
+# ifdef INT_MAX
+#  define	MAXINT		INT_MAX
+# else
+#  define	MAXINT		(1 << 30)
+# endif
+#endif
+
 #if !defined(DEFAULT_ENCODING)
 # define        DEFAULT_ENCODING  0
 #endif
 
+#define MAXJARG         100
+
 /*
  * to turn on debugging, define the preprocessor macro DEBUG and set
  * the variable Debug_flag to the sum of the sections to debug.
@@ -264,17 +277,18 @@ extern int opt_indent;		/* starting column for text printing */
 extern int opt_lines;		/* number of lines to fit on an reduced page */
 extern int opt_killtrail;       /* stop reading input on %%TRailer */
 extern int opt_width;		/* number of columns to fit on reduced page */
-extern int opt_mp_header;	/* let mapge create a header */
-extern int  opt_sheetheader;    /* let mpage create sheetheaders */
+extern int opt_mp_header;	/* let mpage create a header */
+extern int opt_sheetheader;     /* let mpage create sheetheaders */
 extern char * opt_page;		/* sheets size: a4 or us letter */
 extern int opt_fold;		/* fold long lines */
 extern int opt_outline;		/* print a nice outline around pages */
 extern int opt_verbose;		/* print a count of pages sent to printer */
 extern int opt_square;		/* Make pages with same aspect as sheets */
 extern int opt_reverse;		/* Print sheets in reverse order */
-extern int opt_first;		/* First sheet # to print, 1 = first */
-extern int opt_last;		/* Last sheet # to print, 0 = EOF */
-extern int opt_alt;		/* Print every Nth sheet */
+extern int opt_jarg;		/* Number of -j arg sets */
+extern int opt_first[MAXJARG];	/* First sheet # to print, 1 = first */
+extern int opt_last[MAXJARG];	/* Last sheet # to print, 0 = EOF */
+extern int opt_alt[MAXJARG];	/* Print every Nth sheet */
 extern int opt_file;            /* should each file appera on a new sheet */
 extern int opt_duplex;          /* duplex mode*/
 extern int opt_tumble;          /* tumble every second pages */
@@ -302,7 +316,8 @@ extern char * sheethead;	/* Header for each physical page */
  * various global information
  */
 extern char MPAGE[];		/* program name */
-extern int ps_pagenum;		/* current page count */
+extern int ps_pagenum;		/* current page count (printed or not) */
+extern int ps_outpages;		/* pages printed */
 extern int had_ps;              /* did we process ps files */
 extern int first_encoding;	/* first encoding in character set */
 extern int last_encoding;	/* last encoding in character set */
@@ -340,6 +355,9 @@ int ps_check();
 void do_ps_doc();
 /* text.c */
 void do_text_doc();
+/* util.c */
+void memgets_init();
+char *memgets();
 
 /*
  * For OS/2
@@ -347,3 +365,13 @@ void do_text_doc();
 #ifdef __EMX__
 # define strcasecmp stricmp
 #endif
+
+/*
+ * For Amiga
+ */
+#ifdef AMIGA
+# define popen      fopen
+# define pclose     fclose
+# define strcasecmp stricmp
+#endif
+
