@@ -6,15 +6,8 @@
  * mpage:    a program to reduce pages of print so that several pages
  *           of output appear on one printed page.
  *
- * Written by:
- *   ...!uunet!\                       Mark Hahn, Sr Systems Engineer
- *              >pyrdc!mark            Pyramid Technology Corporation
- * ...!pyramid!/                       Vienna, Va    (703)848-2050
- *
- *
+ * Copyright (c) 1994-1999 Marcel J.E. Mol, The Netherlands
  * Copyright (c) 1988 Mark P. Hahn, Herndon, Virginia
- * Copyright (c) 1994-1997 Marcel J.E. Mol, The Netherlands
- *                    marcel@mesa.nl
  *  
  *     Permission is granted to anyone to make or distribute verbatim
  *     copies of this document as received, in any medium, provided
@@ -56,7 +49,7 @@ static int have_line = 0;
 /*
  * number of output lines on current logical page
  */
-static plcnt;
+static int plcnt;
 
 
 int have_showsheet = 0;
@@ -81,10 +74,15 @@ static void  ps_skip_to_page();
 static int   do_post_sheet();
 static void  ps_sheetsetup();
 static int   post_onepage();
+static int   post_flush_onepage();
+static int   post_one_line();
 static void  do_roff_tailer();
 int   ps_check();
 void  do_ps_doc();
 
+static int post_flush_onepage();
+static int  ps_store_to_page();
+
 /*
  * Peek at the first two chacters on the open file and check for the
  * two character postscript flag "%!".  If the file is not postscript
@@ -250,7 +248,7 @@ ps_gettype(fd, outfd)
                 ps_type = PS_MSWINDOWS;
             }
             else if (!strncmp(currline+11, "dvips", 5)) {
-                Debug(DB_PSDOC, "%%ps_gettype: idvips\n", 0);
+                Debug(DB_PSDOC, "%%ps_gettype: dvips\n", 0);
                 ps_type = PS_TEX;
             }
         }
@@ -358,7 +356,11 @@ ps_copyprolog(fd, outfd)
 
     Debug(DB_PSDOC, "%%ps_copyprolog: adding mpage prolog\n", 0);
     if (!have_showsheet) {
+#if 1
+        fprintf(outfd, "/showsheet  /showpage load def\n");
+#else
         fprintf(outfd, "/showsheet { showpage } bind def\n");
+#endif
         fprintf(outfd, "/showpage { } def\n");
         have_showsheet = 1;
     }
@@ -490,6 +492,40 @@ ps_skip_to_page(fd)
 
 } /* ps_skip_to_page */
 
+
+
+static char *stored_page = NULL;
+static int stored_page_size = 0;
+
+static int
+ps_store_to_page(fd)
+ FILE *fd;
+{
+    int totlen = 0;
+
+    Debug(DB_PSDOC, "%%ps_store to page: reading until %%%%Page:\n", 0);
+    while(fgets(currline, LINESIZE-1, fd) != NULL) {
+        int len;
+        if (strncmp(currline, "%%Page:", 7) == 0)
+            return totlen;
+        len = strlen(currline);
+        totlen += len;
+        if (totlen > stored_page_size){
+            stored_page = realloc(stored_page, totlen);
+            stored_page_size = totlen;
+        }
+        /* do not copy the '\0' */
+        memcpy(stored_page + totlen - len, currline, len); 
+        Debug(DB_PSDOC, "%% %s", currline);
+    }
+    Debug(DB_PSDOC, "%%ps_store_to_page: eof before %%%%Page:\n", 0);
+
+    return totlen;
+
+} /* ps_store_to_page */
+
+
+
 /* GPN */
 /* #define NSCALE	to take care of previous scaling */
 #ifdef NSCALE
@@ -520,10 +556,13 @@ do_post_sheet(fd, asheet, outfd)
 {
     int rtn_val = FILE_MORE;
     int sh_high, sh_wide;
+    struct pagepoints *stored_points = NULL, *flush_points = NULL;
+    int flush = 0, totlen = 0;
 
-    if ((points->pp_origin_x == 0 && !points->skip) || opt_file) {
+    do {
+      if ((points->pp_origin_x == 0 && !points->skip) || opt_file || flush) {
         /*
-         * keep track of the pages printed
+         * keep track of the pages processed
          */
         ps_pagenum++;
         fprintf(outfd, "%%%%Page: %d %d\n", ps_pagenum, ps_pagenum);
@@ -532,7 +571,7 @@ do_post_sheet(fd, asheet, outfd)
             fprintf(outfd, "(Page: %d\\n) print flush\n", ps_pagenum);
 # endif /* DEBUG */
 #if 0
-     /* seems to give memory problems... */
+        /* seems to give memory problems... */
         fprintf(outfd, "/sheetsave save def\n");
 #endif
 
@@ -551,16 +590,196 @@ do_post_sheet(fd, asheet, outfd)
          * run through the list of base points for putting reduced pages
          * on the printed page
          */
-        points = asheet->sh_pagepoints;
-    }
-    while ((points->pp_origin_x!=0 || points->skip) && rtn_val == FILE_MORE) {
+        if (!flush)
+            points = asheet->sh_pagepoints;
+        else
+            points++;
+      }
+
+      while ((points->pp_origin_x!=0 || points->skip) && rtn_val == FILE_MORE) {
 
         /* GPN. skip this page ?*/
-     	 if (points->skip!=0)   {
-             ps_skip_to_page(fd);
-             points++;
-             continue;
-     	 }
+        if (points->skip!=0)   {
+            switch (points->skip) {
+                case SKIP_PS:
+                  /*
+                   * Skip this page. User needs another run with the other
+                   * of -O/-E option.
+                   */
+                  ps_skip_to_page(fd);
+                  points++;
+                  continue;
+                case STORE_PS:
+                  /*
+                   *  stored_page could also be an array
+                   *  if more than one page
+                   *  is needed
+                   */
+                  totlen = ps_store_to_page(fd);
+                  stored_points = points;
+                  flush = STORE_PS;
+                  points++;
+                  continue;
+                case FLUSH_PS:
+                  if (stored_points) {
+                      /*
+                       * Ok I have to print the page and start a new one.
+                       * This could be more elegant, but for now...
+                       */
+                      /*
+                      fprintf(outfd, "showsheet\n");
+                      ps_pagenum++;
+                      fprintf(outfd, "%%%%Page: %d %d\n",
+                                     ps_pagenum, ps_pagenum);
+                      sheetheader(outfd, file_name);
+                      mp_outline(outfd, asheet);
+                      */
+                      flush = FLUSH_PS;
+                      flush_points = points;
+                      points = stored_points;
+                  }
+                  else
+                      flush = 0;
+            }
+        }
+
+        /*
+         * Save current graphics context so we can scale/translate
+         * from known position and size.
+         */
+        fprintf(outfd, "gsave\n");
+
+        /*
+         * print one reduced page by moving to the proper point,
+         * turning to the proper aspect, scaling to the proper
+         * size, and setting up a clip path to prevent overwritting;
+         * then print a reduced page of output.
+         */
+        Debug(DB_PSMPAGE, "%%%% %%%%ReducedPageStartsHere\n", outfd);
+# ifdef DEBUG
+        if (Debug_flag & DB_PSMPAGE) {
+            fprintf(outfd, "(    %d %d translate %d rotate\\n)",
+                            points->pp_origin_x(), points->pp_origin_y(),
+                            asheet->sh_rotate);
+            fprintf(outfd, " print flush\n");
+        }
+# endif /* DEBUG */
+
+#ifdef  NSCALE /*GPN*/
+        fprintf (outfd, NScale);
+
+        fprintf(outfd, "%d gpnxs mul %d gpnxs mul translate %d rotate\n",
+                      points->pp_origin_x(), points->pp_origin_y(),
+   	     	      	                           asheet->sh_rotate);
+#else /* NSCALE */
+        fprintf(outfd, "%d %d translate\n",
+                       points->pp_origin_x(), points->pp_origin_y());
+   	if (asheet->sh_rotate)
+            fprintf(outfd, "%d rotate\n", asheet->sh_rotate);
+
+#endif
+        sh_wide = (*asheet->sh_width)();
+        sh_high = (*asheet->sh_height)();
+
+        /* output the clip path */
+#ifdef  NSCALE /*GPN*/
+        fprintf(outfd, "0 0 moveto 0 %d gpnys mul lineto %d gpnxs mul"
+                       " %d gpnys mul lineto ",
+       	       	    sh_high, sh_wide, sh_high);
+        fprintf(outfd, "%d gpnxs mul 0 lineto\n", sh_wide);
+   	fprintf(outfd, "closepath clip\n");
+#else /* NSCALE */
+#if 0
+        fprintf(outfd, "0 0 moveto 0 %d lineto %d %d lineto ",
+                       sh_high, sh_wide, sh_high);
+        fprintf(outfd, "%d 0 lineto\n", sh_wide);
+#else
+        fprintf(outfd, "1 1 moveto 1 %d lineto %d %d lineto ",
+                       sh_high - 1, sh_wide - 1, sh_high - 1);
+        fprintf(outfd, "%d 1 lineto\n", sh_wide - 1);
+#endif
+        fprintf(outfd, "closepath clip newpath\n");
+#endif /* NSCALE */
+
+        if (opt_square) {
+            int newhigh = sh_high, newwide = sh_wide;
+
+            if (sh_wide * ps_height > sh_high * ps_width)
+                newwide = (sh_high * ps_width) / ps_height;
+            else
+                newhigh = (sh_wide * ps_height) / ps_width;
+
+#ifdef  NSCALE /*GPN*/
+    	    fprintf(outfd, "%d gpnxs mul %d gpnys mul translate\n",
+#else
+            fprintf(outfd, "%d %d translate\n",
+#endif
+
+                           (sh_wide - newwide) / 2, (sh_high - newhigh) / 2);
+            sh_wide = newwide;
+            sh_high = newhigh;
+   	}
+
+        fprintf(outfd, "%d %d div %d %d div scale\n",
+                       sh_wide, ps_width, sh_high, ps_height);
+
+        /*
+         * Take extra page margins into account
+         */
+        fprintf(outfd, "%d %d div %d %d div scale\n",
+               ps_width - pagemargin_left - pagemargin_right, ps_width,
+               ps_height - pagemargin_top - pagemargin_bottom, ps_height);
+#ifndef NSCALE
+        fprintf(outfd, "%d %d translate\n", pagemargin_left, pagemargin_bottom);
+#endif
+
+#if 0
+        /* does not seem to be neccessary. seems to create dictionary
+         * stack underflows...
+         */
+        if ((ps_posttype == PS_TEX || ps_posttype == PS_TEX2) && tex1)
+            /* start dvi2ps init every page */
+	    fprintf(outfd, "%s%s", tex1, tex2);
+#endif
+
+        /*
+         * do the individual sheet setup
+         */
+        ps_sheetsetup(outfd);
+        /*
+         * place one reduce page on the printed page
+         */
+        plcnt = 0;
+        if (flush == FLUSH_PS) {
+            rtn_val = post_flush_onepage(totlen, asheet, outfd);
+            stored_points = NULL;
+            points = flush_points;
+        }
+        else {
+            rtn_val = post_onepage(fd, asheet, outfd);
+            points++;
+        }
+        /*
+         * clean up after mpage has drawn its page
+         */
+        fprintf(outfd, "grestore\n");
+      }
+#if 0
+      fprintf(outfd, "sheetsave restore\n");
+#endif
+
+      /*
+       * print the sheet
+       */
+      if (points->pp_origin_x == 0 || (rtn_val == FILE_EOF && opt_file))
+          fprintf(outfd, "showsheet\n");
+
+    } while (flush && rtn_val == FILE_MORE);
+
+    if (stored_points) {
+ 
+        flush_points = points;
+        points = stored_points;
 
         /*
          * Save current graphics context so we can scale/translate
@@ -589,21 +808,35 @@ do_post_sheet(fd, asheet, outfd)
         fprintf(outfd, "%d gpnxs mul %d gpnxs mul translate %d rotate\n",
                       points->pp_origin_x(), points->pp_origin_y(),
    	     	      	                           asheet->sh_rotate);
-
-        sh_wide = (*asheet->sh_width)();
-        sh_high = (*asheet->sh_height)();
 #else /* NSCALE */
         fprintf(outfd, "%d %d translate\n",
                        points->pp_origin_x(), points->pp_origin_y());
    	if (asheet->sh_rotate)
             fprintf(outfd, "%d rotate\n", asheet->sh_rotate);
 
-        /*
-         * Take extra page margins into account
-         */
-        sh_wide = (*asheet->sh_width)() - pagemargin_left - pagemargin_right;
-        sh_high = (*asheet->sh_height)() - pagemargin_top - pagemargin_bottom;
 #endif
+        sh_wide = (*asheet->sh_width)();
+        sh_high = (*asheet->sh_height)();
+
+        /* output the clip path */
+#ifdef  NSCALE /*GPN*/
+        fprintf(outfd, "0 0 moveto 0 %d gpnys mul lineto %d gpnxs mul"
+                       " %d gpnys mul lineto ",
+       	       	    sh_high, sh_wide, sh_high);
+        fprintf(outfd, "%d gpnxs mul 0 lineto\n", sh_wide);
+   	fprintf(outfd, "closepath clip\n");
+#else /* NSCALE */
+#if 0
+        fprintf(outfd, "0 0 moveto 0 %d lineto %d %d lineto ",
+                       sh_high, sh_wide, sh_high);
+        fprintf(outfd, "%d 0 lineto\n", sh_wide);
+#else
+        fprintf(outfd, "1 1 moveto 1 %d lineto %d %d lineto ",
+                       sh_high - 1, sh_wide - 1, sh_high - 1);
+        fprintf(outfd, "%d 1 lineto\n", sh_wide - 1);
+#endif
+        fprintf(outfd, "closepath clip newpath\n");
+#endif /* NSCALE */
 
         if (opt_square) {
             int newhigh = sh_high, newwide = sh_wide;
@@ -624,34 +857,23 @@ do_post_sheet(fd, asheet, outfd)
             sh_high = newhigh;
    	}
 
+        fprintf(outfd, "%d %d div %d %d div scale\n",
+                       sh_wide, ps_width, sh_high, ps_height);
+
+        /*
+         * Take extra page margins into account
+         */
+        fprintf(outfd, "%d %d div %d %d div scale\n",
+               ps_width - pagemargin_left - pagemargin_right, ps_width,
+               ps_height - pagemargin_top - pagemargin_bottom, ps_height);
 #ifndef NSCALE
         fprintf(outfd, "%d %d translate\n", pagemargin_left, pagemargin_bottom);
 #endif
 
-        fprintf(outfd, "%d %d div %d %d div scale\n",
-                       sh_wide, ps_width, sh_high, ps_height);
-        /* output the clip path */
-
-#ifdef  NSCALE /*GPN*/
-        fprintf(outfd, "0 0 moveto 0 %d gpnys mul lineto %d gpnxs mul"
-                       " %d gpnys mul lineto ",
-       	       	    ps_height, ps_width, ps_height);
-        fprintf(outfd, "%d gpnxs mul 0 lineto\n", ps_width);
-#else /* NSCALE */
-        fprintf(outfd, "0 0 moveto 0 %d lineto %d %d lineto ",
-                       ps_height, ps_width, ps_height);
-        fprintf(outfd, "%d 0 lineto\n", ps_width);
-#endif
-
-#ifdef  NSCALE /*GPN*/
-   	fprintf(outfd, "closepath clip\n");
-#else
-        fprintf(outfd, "closepath clip newpath\n");
-#endif /* NSCALE */
 #if 0
-   /* does not seem to be neccessary. seems to create dictionary
-      stack underflows...
-    */
+        /* does not seem to be neccessary. seems to create dictionary
+         * stack underflows...
+         */
         if ((ps_posttype == PS_TEX || ps_posttype == PS_TEX2) && tex1)
             /* start dvi2ps init every page */
 	    fprintf(outfd, "%s%s", tex1, tex2);
@@ -665,22 +887,23 @@ do_post_sheet(fd, asheet, outfd)
          * place one reduce page on the printed page
          */
         plcnt = 0;
-        rtn_val = post_onepage(fd, asheet, outfd);
+        rtn_val = post_flush_onepage(totlen, asheet, outfd);
+        stored_points = NULL;
+
         /*
          * clean up after mpage as drawn its page
          */
         fprintf(outfd, "grestore\n");
-        points++;
-    }
 #if 0
-    fprintf(outfd, "sheetsave restore\n");
+        fprintf(outfd, "sheetsave restore\n");
 #endif
 
-    /*
-     * print the sheet
-     */
-    if (points->pp_origin_x == 0 || (rtn_val == FILE_EOF && opt_file))
-        fprintf(outfd, "showsheet\n");
+        /*
+         * print the sheet
+         */
+        if (points->pp_origin_x == 0 || (rtn_val == FILE_EOF && opt_file))
+            fprintf(outfd, "showsheet\n");
+      }
 
     /*
      * let the upper level know about the status of possible EOF
@@ -718,6 +941,7 @@ post_onepage(fd, asheet, outfd)
  struct sheet *asheet;
  FILE *outfd;
 {
+    int indoc = 0;
 
     Debug(DB_PSROFF, "%%post_onepage: Begin page\n", 0);
     if (ps_at_trailer) {
@@ -731,60 +955,10 @@ post_onepage(fd, asheet, outfd)
     }
         
     while(fgets(currline, LINESIZE-1, fd) != NULL) {
-        if (strncmp(currline, "%%Page:", 7) == 0) {
-            fprintf(outfd, "%% %s", currline);
-            /* only if there is already output to this logical page */
-            if (!plcnt)
-                continue;
-            Debug(DB_PSROFF, "%%post_onepage: next page\n", 0);
-            return FILE_MORE;
-        }
-        if (opt_killtrail && (strncmp(currline, "%%Trailer", 9) == 0 ||
-            strncmp(currline, "%%PSTrailer", 11) == 0)) {
-            fprintf(outfd, "%% %s", currline);
-            Debug(DB_PSROFF, "%%post_onepage: found trailer\n", 0);
-            ps_at_trailer = TRUE;
-            return FILE_EOF;
-        }
-        /* For netscape output */
-        if (strncmp(currline, "%%EOF", 5) == 0) {
-            fprintf(outfd, "%% %s", currline);
-            Debug(DB_PSROFF, "%%post_onepage: EOF\n", 0);
-            return FILE_EOF;
-        }
-        if (ps_posttype == PS_MPAGE && strncmp(currline, "showsheet", 9) == 0)
-            continue;
-        if (ps_posttype == PS_TEX || ps_posttype == PS_TEX2) {
-            if (ps_posttype == PS_TEX2 &&
-                (strstr(currline, "eop\n") || strstr(currline, "eop end\n"))) {
-                fprintf(outfd, "%s", currline);
-                Debug(DB_PSROFF, "%%post_onepage: found TEX eop\n", 0);
-                return FILE_MORE;
-            }
-            if (strncmp(currline, "TeXDict", 7) == 0) {
-                /*
-                 * Hope all dvi2ps progs work the same:
-                 * capture the TeX init code so we can run it 'manually' for
-                 * every page. This is needed as this code sets up a gstate 
-                 * that conflicts with mpage...
-                 * This trick seems to work for text, but figures within the dvi
-                 * file seem to have a mind of their own...
-                 */
-                if (tex1)
-                    free(tex1);
-                tex1 = malloc(strlen(currline)+1);
-                strcpy(tex1, currline);
-                fprintf(outfd, "%s", currline);
-
-                fgets(currline, LINESIZE-1, fd);
-                if (tex2)
-                    free(tex2);
-                tex2 = malloc(strlen(currline)+1);
-                strcpy(tex2, currline);
-            }
-        }
-        fprintf(outfd, "%s", currline);
-        plcnt++;
+        int line_rc;
+        line_rc = post_one_line(currline, fd, outfd, &indoc, 0);
+        if (line_rc != FILE_CONT)
+            return line_rc;
     }
     Debug(DB_PSROFF, "%%post_onepage: eof\n", 0);
 
@@ -794,6 +968,122 @@ post_onepage(fd, asheet, outfd)
 
 
 
+static int
+post_flush_onepage(totlen, asheet, outfd)
+ int totlen;
+ struct sheet *asheet;
+ FILE *outfd;
+{
+    int indoc = 0;
+
+    Debug(DB_PSROFF, "%%post_flush_onepage: Begin page\n", 0);
+
+    /*
+    if (ps_at_trailer) {
+        Debug(DB_PSROFF, "%%post_flush_onepage: still at trailer\n", 0);
+        return FILE_EOF;
+    }
+    */
+
+    if (have_line) {
+        fprintf(outfd, "%s", currline);
+        have_line = 0;
+    }
+        
+    memgets_init(stored_page, totlen);
+    while (memgets(currline, LINESIZE-1) != NULL) {
+        int line_rc;
+        line_rc = post_one_line(currline, (FILE *) NULL, outfd, &indoc, 1);
+        if (line_rc != FILE_CONT)
+            return line_rc;
+    }
+    Debug(DB_PSROFF, "%%post_flush_onepage: eof\n", 0);
+
+    return FILE_MORE;
+
+} /* post_flush_onepage */
+
+
+
+static int
+post_one_line(line, fd, outfd, indoc, flush_page)
+ char * line;
+ FILE *fd;
+ FILE *outfd;
+ int * indoc;
+ int flush_page;
+{
+        if (strncmp(line, "%%BeginDocument", 15) == 0) {
+            (*indoc)++;
+        }
+        if (strncmp(line, "%%EndDocument", 13) == 0) {
+            (*indoc)--;
+        }
+	if (!*indoc) {
+            if (strncmp(line, "%%Page:", 7) == 0) {
+                fprintf(outfd, "%% %s", line);
+                /*
+                 * only if there is already output to this logical page
+                 */
+                if (!plcnt)
+                    return FILE_CONT;
+                Debug(DB_PSROFF, "%%post_one_line: next page\n", 0);
+                return FILE_MORE;
+            }
+            if (opt_killtrail && (strncmp(line, "%%Trailer", 9) == 0 ||
+                strncmp(line, "%%PSTrailer", 11) == 0)) {
+                fprintf(outfd, "%% %s", line);
+                Debug(DB_PSROFF, "%%post_one_line: found trailer\n", 0);
+                ps_at_trailer = TRUE;
+                return FILE_EOF;
+            }
+            /* For netscape output */
+            if (strncmp(line, "%%EOF", 5) == 0) {
+                fprintf(outfd, "%% %s", line);
+                Debug(DB_PSROFF, "%%post_one_line: Netscape EOF\n", 0);
+                return FILE_EOF;
+            }
+            if (ps_posttype == PS_MPAGE && strncmp(line, "showsheet", 9) == 0)
+                return FILE_CONT;
+            if (ps_posttype == PS_TEX || ps_posttype == PS_TEX2) {
+                if (ps_posttype == PS_TEX2 &&
+                    (strstr(line, "eop\n") || strstr(line, "eop end\n"))) {
+                    fprintf(outfd, "%s", line);
+                    Debug(DB_PSROFF, "%%post_one_line: found TEX eop\n", 0);
+                    return FILE_MORE;
+                }
+                if (strncmp(line, "TeXDict", 7) == 0) {
+                    /*
+                     * Hope all dvi2ps progs work the same:
+                     * capture the TeX init code so we can run it 'manually' for
+                     * every page. This is needed as this code sets up a gstate 
+                     * that conflicts with mpage...
+                     * This trick seems to work for text, but figures within the dvi
+                     * file seem to have a mind of their own...
+                     */
+                    if (tex1)
+                        free(tex1);
+                    tex1 = malloc(strlen(line)+1);
+                    strcpy(tex1, line);
+                    fprintf(outfd, "%s", line);
+                    flush_page ?  memgets(line, LINESIZE-1) :
+                                  fgets(line, LINESIZE-1, fd);
+                    if (tex2)
+                        free(tex2);
+                    tex2 = malloc(strlen(line)+1);
+                    strcpy(tex2, line);
+                } 
+            }
+	}
+        fprintf(outfd, "%s", line);
+        plcnt++;
+
+        return FILE_CONT;
+
+} /* post_one_line */
+
+
+
 static void
 do_roff_tailer(fd, outfd)
  FILE *fd, *outfd;
